%% Solve dual QP using generic interior point solver


function [x, history,results,dof] = gtf2d_dualQP_gurobi( y,sz,k,lambda)

%%  sz is the size of the image


n=length(y);

n1=sz(1);
n2=sz(2);

R=reshape(y,[n1,n2]);

D1dx = diff(speye(n1));
D1dy = diff(speye(n2));
D1 = [kron(speye(n2),D1dx); kron(D1dy,speye(n1))];

Delta=construct_O(D1,k);


objfun = @(theta) 0.5*norm(y-theta)^2 + lambda*sum(abs(Delta*theta));

m=size(Delta,1);


clear model;
model.Q= Delta*Delta' + 50*eps*speye(m);  
% the additional term added to the diagonal is to ensure the numerically
% this matrix is PSD.  Gurobi complains if I do not add something like
% this.
model.obj=-2*full(Delta*y)';
model.A=sparse(0,m);
model.rhs=[];
model.sense='';
model.lb=-lambda*ones(m,1);
model.ub= lambda*ones(m,1);


param.method=2;
results = gurobi(model,param);



if strcmp(results.status,'OPTIMAL')
    fprintf('Optimal objective: %e\n', results.objval);
else
    fprintf('[Warning] Gurobi failure. Status: %s.\n',results.status);
end

u=results.x;
% construct the primal solution.
x= y - Delta'*u;

history = objfun(x);
dof  = sum(abs(Delta*x) < 1e-10);

end
