function [ x, history ] = gtf_admm_grid_v1( y,sz,k,lambda, rho, varargin )
%GTF_ADMM_grid_v1 Summary of this function goes here

%   ADMM version of the graph trend filtering
% for specialized graphs (2D grid or 3d grid)
% lambda is tuning parameter, rho is the numerical parameter
% The difference from "gtf_admm_grid" is that we use only
% soft-thresholding.
%
% 
% k is the order.

%the k notation is different from signal processing literature by 1
%k=0:  O=D
%k=1:  O=L
%k=2:  O=DL


n=length(y);

if n ~= prod(sz)
    fprintf('size of input needs to be consistent!')
    return;
end


if length(sz)==2
    D = get_Delta_grid(sz,'gtf2d',0);
elseif length(sz)==3
    D = get_Delta_grid(sz,'gtf3d',0);
else
    fprintf('Grids with dimension  > 3 not supported');
    return;
end

%[edges1,edges2] = incidence2edgelist(D);

if nargin == 6
    idx=varargin{1};
else
    i=(1-mod(k,2))+1;
    idx=false(size(D,i),1);
end


O=construct_O(D,k);

tol_abs=1e-5;
tol_rel=1e-4;

O=O(~idx,:);



%L=chol(rho*(O'*O)+speye(n),'lower');

% initialization
conv=0;
x=y;
m=size(O,1);
z=zeros(size(O,1),1);
u=z;
iter = 1;
maxiter=1000;
history=[];
while ~conv
   if length(sz)==2
      x = grid_system(reshape((O'*(rho*z-u)+y),sz),(k+1)/2*2,rho);
   elseif length(sz)==3
      x = grid_system_3d(reshape((O'*(rho*z-u)+y),sz),(k+1)/2*2,rho);
      % Check that the solution is the same as below
      %x=x(:);
      %[x1,pcg_flg]=pcg(A,(Lk*(rho*z-u)+y),tol_pcg,k*100);
   end
    x=x(:);   
       
       
    Ox=O*x;
    z_new=soft_thresh(Ox+u/rho,lambda/rho);
    s= rho*norm(O'*(z_new-z));
    z=z_new;
    
    u=u+rho*(Ox-z);
    r=norm(Ox-z);
    
    eps_pri= sqrt(n)*tol_abs + tol_rel*max(norm(Ox),norm(z));    
    eps_dual= sqrt(m)*tol_abs  + tol_rel*norm(O'*u);
   
    if ~mod(iter,50)
        fprintf('[%d] [r,s]=%f, %f, [eps_pri, eps_dual]=%f, %f. \n',...
            iter,r,s, eps_pri,eps_dual);
    end

    if r < eps_pri && s< eps_dual
        conv=1;
        fprintf('converged.\n')
    elseif iter >= maxiter
        conv=1;
        fprintf('Reached maxiter.\n')
    end
    
    history=[history,[s;r]];
    
        tau=2;
    if r>10*s
        rho=rho*tau;
    elseif s>10*r
        rho=rho/2;
    end
    
    iter=iter+1;
end


end

